﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using UnityEngine;
using UnityEditor;
using UnityEditor.Events;
using UnityEditor.SceneManagement;
using UnityEngine.Events;
using UnityEngine.EventSystems;
using UnityEngine.SceneManagement;
using Object = UnityEngine.Object;

public class InteractableSetup : EditorWindow
{
	[MenuItem("Adventure Game/Complete DoorToSecurityRoom")]
	public static void SetupDoorToSecurityRoom()
	{
		if (SceneManager.GetSceneAt(0).name != "Market")
		{
			Debug.LogError("The Market scene is not the primary loaded scene.  No Reactions will be edited.");
			return;
		}

		AudioSource fxAudioSource = FindReference<AudioSource> ("FX");

		if (!fxAudioSource)
		{
			return;
		}

		GameObject doorToSecurityRoom = GameObject.Find("DoorToSecurityRoomInteractable");

		Interactable interactable = doorToSecurityRoom.GetComponent<Interactable>();

		if (!SetupEventTrigger(interactable))
		{
			Debug.LogError("The EventTrigger of the DoorToSecurityRoomInteractable gameobject was not successfully added to.");
		}

		Reaction reaction = interactable.transform.Find("DefaultReaction").GetComponent<Reaction>();

		PersistentDataObject playerLocationData = AssetDatabase.LoadAssetAtPath<PersistentDataObject>("Assets/PersistentDataObjects/Player/PlayerLocationData.asset");

		if (playerLocationData)
			reaction.sceneReaction.playerLocationPersistentDataObject = playerLocationData;
		else
			Debug.LogError("The PlayerLocationData PersistentDataObject was not found in PersistentDataObjects folder.\n" +
				"Please assign the PlayerLocationData PersistentDataObject to DoorToSecurityRoomInteractable.DefaultReaction.Reaction.SceneReaction.PlayerLocationPersistentDataObject");

		reaction.audioReactions[0].audioSource = fxAudioSource;

		MarkPrefabAsChanged(doorToSecurityRoom);
	}


	private static T FindReference <T> (string gameObjectName)
		where T : Component
	{
		T referenceToFind = null;

		GameObject gameObject = GameObject.Find (gameObjectName);

		if (!gameObject)
		{
			Debug.LogError ("Could not find gameobject called " + gameObjectName);
		}
		else
		{
			referenceToFind = gameObject.GetComponent<T> ();

			if (!referenceToFind)
			{
				Debug.LogError ("Could not find a componet of type " + typeof(T) + " on " + gameObjectName);
			}
		}

		return referenceToFind;
	}


	private static bool SetupEventTrigger(Interactable interactable)
	{
		if (!interactable)
		{
			Debug.LogError("A null interactable was passed for EventTrigger setup.");
			return false;
		}

		GameObject player = GameObject.Find("Player");

		if (!player)
		{
			Debug.LogError("The Player gameobject could not be found.");
			return false;
		}

		var playerMovement = player.GetComponent("PlayerMovement");

		if (!playerMovement)
		{
			Debug.LogError("The Player gameobject does not have a PlayerMovement script.");
			return false;
		}

		EventTrigger eventTrigger = interactable.GetComponent<EventTrigger>();

		if (!eventTrigger)
		{
			Debug.LogError("The " + interactable.name + " gameobject does not have an EventTrigger component.");
			return false;
		}

		EventTrigger.Entry pointerClickEntry = eventTrigger.triggers.FirstOrDefault(x => x.eventID == EventTriggerType.PointerClick);

		if (pointerClickEntry == null)
		{
			Debug.LogError("The first PointerClick event could not be found on the " + eventTrigger.name + " gameobject.");
			return false;
		}

		Type playerMovementType = typeof(Interactable).Assembly.GetType("PlayerMovement");

		if (playerMovementType == null)
		{
			Debug.LogError("A type called PlayerMovement could not be found.");
			return false;
		}

		MethodInfo onInteractableClickMethodInfo = playerMovementType.GetMethod("OnInteractableClick");

		if (onInteractableClickMethodInfo == null)
		{
			Debug.LogError("A method called OnInteractableClick could not be found in PlayerMovement");
			return false;
		}

		UnityAction<Interactable> onInteractableClickReflectedCall = (UnityAction<Interactable>)Delegate.CreateDelegate(typeof(UnityAction<Interactable>), playerMovement, onInteractableClickMethodInfo);

		UnityEventTools.AddObjectPersistentListener(pointerClickEntry.callback, onInteractableClickReflectedCall, interactable);

		return true;
	}


	private static void MarkPrefabAsChanged(GameObject prefab)
	{
		foreach (var c in prefab.GetComponentsInChildren<Component>())
		{
			PrefabUtility.RecordPrefabInstancePropertyModifications(c);
		}
		PrefabUtility.RecordPrefabInstancePropertyModifications(prefab);

		EditorSceneManager.MarkSceneDirty(SceneManager.GetSceneAt(0));
	}
}
